@@ -50,15 +50,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_SCALEFACTOR \
 	MS_FILTER_METHOD(MSFilterVideoDisplayInterface,5,float)
 
-/**scale factor of the local view */
-#define MS_VIDEO_DISPLAY_SET_SELFVIEW_POS \
-	MS_FILTER_METHOD(MSFilterVideoDisplayInterface,6,float[3])
-
-/**scale factor of the local view */
-#define MS_VIDEO_DISPLAY_GET_SELFVIEW_POS \
-	MS_FILTER_METHOD(MSFilterVideoDisplayInterface,7,float[3])
-
-/**scale factor of the local view */
+/**Set the background colour for video window */
 #define MS_VIDEO_DISPLAY_SET_BACKGROUND_COLOR \
 	MS_FILTER_METHOD(MSFilterVideoDisplayInterface,8,int[3])
 
@@ -95,4 +87,42 @@ typedef enum _MSPlayerState MSPlayerState;
 	MS_FILTER_METHOD(MSFilterPlayerInterface,5,int)
 
 
+/** Interface definitions for echo cancellers */
+
+/** sets the echo delay in milliseconds*/
+#define MS_ECHO_CANCELLER_SET_DELAY \
+	MS_FILTER_METHOD(MSFilterEchoCancellerInterface,0,int)
+
+#define MS_ECHO_CANCELLER_SET_FRAMESIZE \
+	MS_FILTER_METHOD(MSFilterEchoCancellerInterface,1,int)
+
+/** sets tail length in milliseconds */
+#define MS_ECHO_CANCELLER_SET_TAIL_LENGTH \
+	MS_FILTER_METHOD(MSFilterEchoCancellerInterface,2,int)
+
+/** put filter in bypass mode */
+#define MS_ECHO_CANCELLER_SET_BYPASS_MODE \
+	MS_FILTER_METHOD(MSFilterEchoCancellerInterface,3,bool_t)
+/** get filter bypass mode */
+#define MS_ECHO_CANCELLER_GET_BYPASS_MODE \
+	MS_FILTER_METHOD(MSFilterEchoCancellerInterface,4,bool_t)
+
+/** retrieve echo canceller internal state, as a base64 encoded string */
+#define MS_ECHO_CANCELLER_GET_STATE_STRING \
+	MS_FILTER_METHOD(MSFilterEchoCancellerInterface,5,char **)
+
+/** restore a previous state suppling the echo canceller config as base64 encoded string */
+#define MS_ECHO_CANCELLER_SET_STATE_STRING \
+	MS_FILTER_METHOD(MSFilterEchoCancellerInterface,6, const char *)
+
+
+
+/** Interface definitions for video decoders */
+#define MS_VIDEO_DECODER_DECODING_ERRORS \
+		MS_FILTER_EVENT_NO_ARG(MSFilterVideoDecoderInterface,0)
+
+/** Interface definitions for video capture */
+#define MS_VIDEO_CAPTURE_SET_DEVICE_ORIENTATION \
+	MS_FILTER_METHOD(MSFilterVideoCaptureInterface,0,int)
+
 #endif
