@@ -71,6 +71,8 @@ static void au_set_source(MSSndCard *card, MSSndCardCapture source)
 {
 }
 
+static bool filter_valid = false;
+
 static void au_init(MSSndCard *card){
 	ms_debug("au_init");
 	AUData *d=ms_new(AUData,1);
@@ -95,9 +97,13 @@ static void au_init(MSSndCard *card){
 		d->is_ringer=FALSE;
 	}
 	card->data=d;
+    
+    filter_valid = TRUE;
 }
 
 static void au_uninit(MSSndCard *card){
+    filter_valid = FALSE;
+    
 	AUData *d=(AUData*)card->data;
 	ms_bufferizer_destroy(d->bufferizer);
 	ms_mutex_destroy(&d->mutex);
@@ -126,12 +132,28 @@ static MSSndCard *au_duplicate(MSSndCard *obj){
 	MSSndCard *card=ms_snd_card_new_with_name(&au_card_desc,obj->name);
 	return card;
 }
+
+static const char *FormatError(OSStatus error)
+{
+	// not re-entrant but..
+	static char str[64];
+
+    // see if it appears to be a 4-char-code
+    *(UInt32 *)(str + 1) = CFSwapInt32HostToBig(error);
+    if (isprint(str[1]) && isprint(str[2]) && isprint(str[3]) && isprint(str[4])) {
+        str[0] = str[5] = '\'';
+        str[6] = '\0';
+    } else
+        // no, format it as an integer
+        sprintf(str, "%d", (int)error);
+    return str;
+}
+
+
 #define check_auresult(au,method) \
-if (au!=0) ms_error("AudioUnit error for %s: ret=%i",method,au)
-
-#ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
-
+if (au!=0) ms_error("AudioUnit error for %s: ret=%s (%li) ",method, FormatError(au), au)
 static void au_interuption_listener(void* inClientData, UInt32 inInterruptionState) {
+    if (!filter_valid) return;
 	if (((MSSndCard*)inClientData)->data == NULL) return;
 	
 	AUData *d=(AUData*)(((MSSndCard*)inClientData)->data);
@@ -152,35 +174,25 @@ static void au_interuption_listener(void* inClientData, UInt32 inInterruptionSta
 			d->io_unit_must_be_started=TRUE;
 			break;
 		default:
-			ms_warning ("unexpected interuption %i",inInterruptionState);
+			ms_warning ("unexpected interuption %li",inInterruptionState);
 	}
 }
 
-#endif
-
 static MSSndCard *au_card_new(const char* name){
 	MSSndCard *card=ms_snd_card_new_with_name(&au_card_desc,name);
-#ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
 	OSStatus auresult = AudioSessionInitialize(NULL, NULL, au_interuption_listener, card);
 	if (auresult != kAudioSessionAlreadyInitialized) {
 		check_auresult(auresult,"AudioSessionInitialize");
 	}
-#endif
 	return card;
 }
 
 static void au_detect(MSSndCardManager *m){
-	MSSndCard *card;
 	ms_debug("au_detect");
-#ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
-	card=au_card_new(AU_CARD_RECEIVER);
-	ms_snd_card_manager_add_card(m,card);
-	card=au_card_new(AU_CARD_SPEAKER);
-	
-#else
-	card=au_card_new("Audio Unit");
-#endif	
+	MSSndCard *card=au_card_new(AU_CARD_RECEIVER);
 	ms_snd_card_manager_add_card(m,card);
+	//card=au_card_new(AU_CARD_SPEAKER); //Disabled because iounit cannot be used in play only mode on IOS 5.0
+	//ms_snd_card_manager_add_card(m,card);	
 }
 
 static OSStatus au_read_cb (
@@ -210,7 +222,7 @@ static OSStatus au_read_cb (
 			putq(&d->rq,rm);
 			ms_mutex_unlock(&d->mutex);
 			d->readTimeStamp.mSampleTime+=ioData->mBuffers[0].mDataByteSize/(d->bits/2);
-		}else ms_warning("AudioUnitRender() failed: %i",err);
+		}else ms_warning("AudioUnitRender() failed: %s (%li)", FormatError(err), err);
 	}
 	return err;
 }
@@ -273,39 +285,37 @@ static void au_configure(AUData *d) {
 	AudioComponent foundComponent;
 	OSStatus auresult;
 	UInt32 doSetProperty      = 1;
-	UInt32 doNotSetProperty    = 0;
+	UInt32 doNotSetProperty    = 0;	
 	
-	
-#ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
 	auresult = AudioSessionSetActive(true);
-	check_auresult(auresult,"AudioSessionSetActive");
+	check_auresult(auresult,"AudioSessionSetActive(true)");
 	
+	UInt32 audioCategory;
 	
-	UInt32 audioCategory =kAudioSessionCategory_PlayAndRecord;
+	if (d->is_ringer && kCFCoreFoundationVersionNumber > kCFCoreFoundationVersionNumber10_6 /*I.E is >=OS4*/) {
+        audioCategory= kAudioSessionCategory_AmbientSound;
+        ms_message("Configuring audio session for play back");
+	} else {
+		audioCategory = kAudioSessionCategory_PlayAndRecord;
+		ms_message("Configuring audio session for play back/record");
+		
+	}
 	auresult =AudioSessionSetProperty(kAudioSessionProperty_AudioCategory, sizeof(audioCategory), &audioCategory);
 	check_auresult(auresult,"Configuring audio session ");
-	
-
-	if (d->is_ringer) {
-		auresult=AudioSessionSetProperty (kAudioSessionProperty_OverrideCategoryDefaultToSpeaker,sizeof (doSetProperty),&doSetProperty);
-		check_auresult(auresult,"kAudioSessionProperty_OverrideAudioRoute");
-		ms_message("Configuring audio session default route to speaker");
-	} else {
-		ms_message("Configuring audio session default route to receiver");
-	}
-#endif
+    if (d->is_ringer && !(kCFCoreFoundationVersionNumber > kCFCoreFoundationVersionNumber10_6 /*I.E is <OS4*/)) {
+        //compatibility with 3.1
+        auresult=AudioSessionSetProperty (kAudioSessionProperty_OverrideCategoryDefaultToSpeaker,sizeof (doSetProperty),&doSetProperty);
+        check_auresult(auresult,"kAudioSessionProperty_OverrideAudioRoute");
+        ms_message("Configuring audio session default route to speaker");            
 
+    }
 	if (d->started == TRUE) {
 		//nothing else to do
 		return;
 	}
 	
 	au_description.componentType          = kAudioUnitType_Output;
-#ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
 	au_description.componentSubType       = kAudioUnitSubType_VoiceProcessingIO;
-#else
-	au_description.componentSubType	      =  kAudioUnitSubType_HALOutput;
-#endif
 	au_description.componentManufacturer  = kAudioUnitManufacturer_Apple;
 	au_description.componentFlags         = 0;
 	au_description.componentFlagsMask     = 0;
@@ -330,27 +340,28 @@ static void au_configure(AUData *d) {
 	
 	check_auresult(auresult,"AudioUnitUninitialize");
 	
-	//read
-	auresult=AudioUnitSetProperty (
-								   d->io_unit,
-								   kAudioOutputUnitProperty_EnableIO,
-								   kAudioUnitScope_Input ,
-								   inputBus,
-								   &doSetProperty,
-								   sizeof (doSetProperty)
-								   );
-	check_auresult(auresult,"kAudioOutputUnitProperty_EnableIO,kAudioUnitScope_Input");
-	//setup stream format
-	auresult=AudioUnitSetProperty (
-								   d->io_unit,
-								   kAudioUnitProperty_StreamFormat,
-								   kAudioUnitScope_Input,
-								   outputBus,
-								   &audioFormat,
-								   sizeof (audioFormat)
-								   );
-	
-	//write	
+if (!d->is_ringer || kCFCoreFoundationVersionNumber <= kCFCoreFoundationVersionNumber10_6 /*I.E is <OS4*/) {
+		//read
+		auresult=AudioUnitSetProperty (
+									   d->io_unit,
+									   kAudioOutputUnitProperty_EnableIO,
+									   kAudioUnitScope_Input ,
+									   inputBus,
+									   &doSetProperty,
+									   sizeof (doSetProperty)
+									   );
+		check_auresult(auresult,"kAudioOutputUnitProperty_EnableIO,kAudioUnitScope_Input");
+		auresult=AudioUnitSetProperty (
+									   d->io_unit,
+									   kAudioUnitProperty_StreamFormat,
+									   kAudioUnitScope_Output,
+									   inputBus,
+									   &audioFormat,
+									   sizeof (audioFormat)
+									   );
+		check_auresult(auresult,"kAudioUnitProperty_StreamFormat,kAudioUnitScope_Output");
+}
+		//write	
 	//enable output bus
 	auresult =AudioUnitSetProperty (
 									d->io_unit,
@@ -366,13 +377,11 @@ static void au_configure(AUData *d) {
 	auresult=AudioUnitSetProperty (
 								   d->io_unit,
 								   kAudioUnitProperty_StreamFormat,
-								   kAudioUnitScope_Output,
-								   inputBus,
+								   kAudioUnitScope_Input,
+								   outputBus,
 								   &audioFormat,
 								   sizeof (audioFormat)
 								   );
-	check_auresult(auresult,"kAudioUnitProperty_StreamFormat,kAudioUnitScope_Output");
-	
 	check_auresult(auresult,"kAudioUnitProperty_StreamFormat,kAudioUnitScope_Input");
 	
 	//disable unit buffer allocation
@@ -401,7 +410,6 @@ static void au_configure(AUData *d) {
 								   );
 	check_auresult(auresult,"kAudioUnitProperty_SetRenderCallback,kAudioUnitScope_Input");
 	
-#ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
 	const Float64 preferredSampleRate = d->rate;//PREFERRED_HW_SAMPLE_RATE; /*optimum to minimize delay, must put a software resampler to deal with 8khz*/
 	
 	auresult=AudioSessionSetProperty(kAudioSessionProperty_PreferredHardwareSampleRate
@@ -410,6 +418,8 @@ static void au_configure(AUData *d) {
 	check_auresult(auresult,"kAudioSessionProperty_PreferredHardwareSampleRate");
 	
 	
+	//start io unit
+	auresult=AudioUnitInitialize (d->io_unit);
 	Float32 preferredBufferSize;
 	switch (d->rate) {
 		case 11025:
@@ -425,8 +435,7 @@ static void au_configure(AUData *d) {
 									 , &preferredBufferSize);
 	
 	
-	if (auresult != 0) ms_message("kAudioSessionProperty_PreferredHardwareIOBufferDuration returns %i ",auresult);
-#endif
+	if (auresult != 0) ms_message("kAudioSessionProperty_PreferredHardwareIOBufferDuration returns %li ",auresult);
 	
 	Float64 delay;
 	UInt32 delaySize = sizeof(delay);
@@ -448,9 +457,7 @@ static void au_configure(AUData *d) {
 	
 	
 	
-	ms_message("I/O unit latency [%f], quality [%i]",delay,quality);
-
-#ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
+	ms_message("I/O unit latency [%f], quality [%li]",delay,quality);
 	Float32 hwoutputlatency;
 	UInt32 hwoutputlatencySize=sizeof(hwoutputlatency);
 	auresult=AudioSessionGetProperty(kAudioSessionProperty_CurrentHardwareOutputLatency
@@ -475,11 +482,6 @@ static void au_configure(AUData *d) {
 									 ,&hwsamplerate);
 
 	ms_message("current hw output latency [%f] input [%f] iobuf[%f] sample rate [%f]",hwoutputlatency,hwinputlatency,hwiobuf,hwsamplerate);
-#else
-
-	auresult=AudioUnitInitialize(d->io_unit);
-	check_auresult(auresult,"AudioUnitInitialize");
-#endif
 	auresult=AudioOutputUnitStart(d->io_unit);
 	check_auresult(auresult,"AudioOutputUnitStart");
 	d->started=TRUE;
@@ -510,6 +512,7 @@ static void au_unconfigure(AUData *d) {
 		AudioOutputUnitStop(d->io_unit);
 		AudioComponentInstanceDispose (d->io_unit);
 		d->started=FALSE;
+		check_auresult(AudioSessionSetActive(false),"AudioSessionSetActive(false)");
 	}
 }
 
@@ -694,5 +697,4 @@ void ms_au_register_card() {
 	 * register audio unit plugin should be move to linphone code
 	 */
 	ms_snd_card_manager_register_desc(ms_snd_card_manager_get(),&au_card_desc);
-}
-
+}	
